package blackholegame.gui;

import blackholegame.logic.BlackHoleGameLogic;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;

public class BlackHoleGameFrame extends JFrame {

    private final BlackHoleGameLogic blackHoleGameLogic;
    private JPanel gamePanel;
    private final ActionListener gameButtonActionListener = new GameButtonActionListener();
    private JToolBar infoPanel;
    private final Action newGameAction = new NewGameAction();
    private JLabel infoLabel;

    public BlackHoleGameFrame(BlackHoleGameLogic blackHoleGameLogic) throws HeadlessException {
        this.blackHoleGameLogic = blackHoleGameLogic;
        blackHoleGameLogic.newGame(9);
        setupUI();
    }

    private void setupUI() {
        setTitle("Blackhole game");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocation(245, 122);
        setLayout(new BorderLayout());
        gamePanel = new JPanel();
        setupGamePanel();
        getContentPane().add(gamePanel, BorderLayout.CENTER);
        infoPanel = new JToolBar();
        infoPanel.add(newGameAction);
        infoLabel = new JLabel();
        infoPanel.add(infoLabel);
        getContentPane().add(infoPanel, BorderLayout.NORTH);
        pack();
        refreshUI();
    }

    private void setupGamePanel() {
        gamePanel.removeAll();
        int n = blackHoleGameLogic.getRowAndColumnCount();
        gamePanel.setLayout(new GridLayout(n, n));
        for (int row = 0; row < n; row++) {
            for (int column = 0; column < n; column++) {
                JButton btn = new GameButton(row, column);
                btn.setPreferredSize(new Dimension(40, 40));
                btn.addActionListener(gameButtonActionListener);
                gamePanel.add(btn);
            }
        }
    }

    private GameButton selectedButton = null;

    private void refreshUI() {
        for (Component component : gamePanel.getComponents()) {
            GameButton btn = (GameButton) component;
            int row = btn.getRow();
            int column = btn.getColumn();
            int field = blackHoleGameLogic.getField(row, column);
            Color buttonColor = Color.WHITE;
            switch (field) {
                case 0:
                    break;
                case 1:
                    buttonColor = Color.ORANGE;
                    break;
                case 2:
                    buttonColor = Color.CYAN;
                    break;
                case -1:
                    buttonColor = Color.BLACK;
                    break;
            }
            if (selectedButton == btn) {
                buttonColor = buttonColor.darker();
            }
            btn.setBackground(buttonColor);
        }
        int actualPlayer = blackHoleGameLogic.getActualPlayer();
        Color infoColor = actualPlayer == 1 ? Color.ORANGE.darker() : Color.CYAN.darker();
        String infoColorHex = colorToHex(infoColor);
        infoLabel.setText("<html><font color=\"#" + infoColorHex + "\">Player " + actualPlayer + "</font> is next.</html>");
    }

    private static String colorToHex(Color color) {
        int red = color.getRed();
        int green = color.getGreen();
        int blue = color.getBlue();
        StringBuilder sb = new StringBuilder();
        sb.append(Integer.toHexString(red / 16));
        sb.append(Integer.toHexString(red % 16));
        sb.append(Integer.toHexString(green / 16));
        sb.append(Integer.toHexString(green % 16));
        sb.append(Integer.toHexString(blue / 16));
        sb.append(Integer.toHexString(blue % 16));
        return sb.toString();
    }

    private class NewGameAction extends AbstractAction {

        public NewGameAction() {
            super("New game");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            Integer[] gameSizes = new Integer[]{5, 7, 9};
            int gameSize = (int) JOptionPane.showInputDialog(rootPane, "Select a new gamesize", "New game", JOptionPane.QUESTION_MESSAGE, null, gameSizes, gameSizes[0]);
            blackHoleGameLogic.newGame(gameSize);
            setupGamePanel();
            pack();
            refreshUI();
        }

    }

    private class GameButtonActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            GameButton btn = (GameButton) e.getSource();
            int row = btn.getRow();
            int column = btn.getColumn();
            if (selectedButton == null) {
                if (blackHoleGameLogic.isShipOfActualPlayer(row, column)) {
                    selectedButton = btn;
                    refreshUI();
                }
            } else {
                blackHoleGameLogic.move(selectedButton.getRow(), selectedButton.getColumn(), row, column);
                selectedButton = null;
                refreshUI();
                checkForEndGame();
            }
        }

        private void checkForEndGame() {
            if (blackHoleGameLogic.playerOneWon()) {
                JOptionPane.showMessageDialog(rootPane, "Player one has won the game", "Grats", JOptionPane.INFORMATION_MESSAGE);
                newGameAction.actionPerformed(null);
            }
            if (blackHoleGameLogic.playerTwoWon()) {
                JOptionPane.showMessageDialog(rootPane, "Player two has won the game", "Grats", JOptionPane.INFORMATION_MESSAGE);
                newGameAction.actionPerformed(null);
            }
        }
    }

}
